import React, { CSSProperties, useEffect, useMemo, useState } from 'react';
import { queryUserListByRoleID, queryLeader } from '@/services/boom';
import { connect } from 'umi';
import { CheckOutlined, PlusOutlined } from '@ant-design/icons';
import { Popover, Radio, RadioChangeEvent, Spin, Steps } from 'antd';
import { useModel, useRequest } from '@umijs/max';

const { Step } = Steps;

const TYPE = {
  ROLE: 'role',
  USER: 'user',
  LEADER: 'leader',
} as const;

interface AuditNode {
  is_cc: 0 | 1;
  seq: number;
  type: (typeof TYPE)[keyof typeof TYPE];
  value: number[];
  checkValue: [];
}

const ApprovalProcess = (props: any) => {
  const {
    approvalProcess,
    leaderData = [],
    dispatch,
    onChange,
    roleList = [],
  } = props;
  const { userList, run } = useModel('userList');
  const [checkValue, setCheckValue] = useState<number[]>([]);

  const { auditList, ccList } = useMemo<{
    auditList: AuditNode[];
    ccList: AuditNode[];
  }>(() => {
    let auditList: AuditNode[] = [],
      ccList: AuditNode[] = [];
    approvalProcess.forEach((item: any, index: number) => {
      if (!item) return;
      let node: AuditNode = {
        is_cc: item[0].is_cc,
        type: item[0].type,
        checkValue: [],
        seq: index,
        value: item.map((node: any) => node.value),
      };

      if (node.is_cc == 1) {
        ccList.push(node);
      } else {
        auditList.push(node);
      }
    });
    return { auditList, ccList };
  }, [approvalProcess]);

  const onCheckValue = (value: number, index: number) => {
    let values = [...checkValue];
    values[index] = value;
    setCheckValue(values);
    onChange(values);
  };

  useEffect(() => {
    dispatch({
      type: 'user/getRoleList',
    });
    run();
  }, []);
  const rowStyle: CSSProperties = {
    display: 'flex',
    flexDirection: 'column',
    fontSize: 16,
  };

  return (
    <div>
      <div style={rowStyle}>
        {auditList.map((item: AuditNode, index: number) => (
          <AuditNodeStep
            key={`${item.type}-${item.value.join('.')}-${index}`}
            leaderData={leaderData}
            item={item}
            roleList={roleList}
            userList={userList}
            value={checkValue[item.seq]}
            onChange={(value: number) => onCheckValue(value, item.seq)}
          />
        ))}
      </div>
      {ccList.length > 0 && (
        <div style={rowStyle}>
          <h3 style={{ margin: "20px 0", fontWeight: 'bold' }}>抄送人</h3>
          {ccList.map((item: AuditNode, index: number) => (
            <AuditNodeStep
              key={`${item.type}-${item.value.join('.')}-${index}`}
              leaderData={leaderData}
              item={item}
              roleList={roleList}
              userList={userList}
              value={checkValue[item.seq]}
              onChange={(value: number) => onCheckValue(value, item.seq)}
            />
          ))}
        </div>
      )}
    </div>
  );
};

interface AuditNodeStepProps {
  item: AuditNode;
  leaderData: any;
  roleList: any;
  userList: any;
  value: number;
  onChange: (value: number) => void;
}

const AuditNodeStep = (props: AuditNodeStepProps) => {
  const {
    item,
    leaderData,
    roleList = [],
    userList = [],
    value,
    onChange,
  } = props;

  const [selectUserList, setSelectUserList] = useState([]);
  const [loading, setLoading] = useState(false);

  const getLeaderContent = (length: any) => (
    <Steps
      progressDot
      current={-1}
      direction="vertical"
      items={leaderData
        ?.slice(0, length || 0)
        .map((leader: any) => ({ title: leader.CName }))}
    ></Steps>
  );

  const handleOpen = async () => {
    setLoading(true);
    if (item.type !== 'role') return;
    const data = await queryUserListByRoleID({ role_id: item.value[0] });
    setSelectUserList(data);
    setLoading(false);
  };

  const selectedUserId = ({ target: { value } }: RadioChangeEvent) => {
    onChange(Number(value));
  };

  const content = (
    <Spin spinning={loading}>
      <Radio.Group onChange={selectedUserId} value={value}>
        {selectUserList.map((item: any) => (
          <Radio.Button value={Number(item.user_id)}>
            {item.c_name}
          </Radio.Button>
        ))}
      </Radio.Group>
    </Spin>
  );

  if (item.type == TYPE.LEADER) {
    return (
      <div style={{ marginBottom: 20 }}>
        <Popover
          placement="bottomLeft"
          title="查看审批人"
          content={getLeaderContent(item.value[0])}
          overlayStyle={{ width: '300px' }}
        >
          <CheckOutlined style={{ marginRight: 20 }} />
          {Math.min(item.value[0], leaderData?.length)}级主管审批
        </Popover>
      </div>
    );
  }
  if (item.type == TYPE.ROLE) {
    let title = '';
    if (value) {
      title = userList.find((cur: any) => cur.ID == value)?.CName;
    } else {
      const names = item.value.map((id) => {
        const role = roleList.find((cur: any) => cur.ID == id);
        return role?.Name;
      });
      title = `从${names.join('、')}选择`;
    }
    return (
      <div style={{ marginBottom: 20, cursor: 'pointer' }}>
        <Popover
          placement="bottomLeft"
          title={'选择审批人'}
          content={content}
          trigger="click"
          overlayStyle={{ width: '300px' }}
          onOpenChange={handleOpen}
          style={{ marginBottom: 20 }}
        >
          <PlusOutlined style={{ marginRight: 20 }} /> {title}
        </Popover>
      </div>
    );
  }
  const names = item.value.map((id) => {
    const user = userList.find((cur: any) => cur.ID == id);
    return user?.CName;
  });

  return (
    <div style={{ marginBottom: 20 }}>
      <CheckOutlined style={{ marginRight: 20 }} /> {names.join('、')}
    </div>
  );
};

export default connect(({ user }: any) => ({
  roleList: user.roleList,
}))(ApprovalProcess);
